package bootstrapP;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.util.List;

/**This interface represents a general backupper, witch may be a file backupper, a remote backupper, a RAID-like backupper, or any other kind of service to permanently store datas.
 * @author Marco Cornolti
 *
 */
public interface BackupperInterface {
	/**Backups the whole data given by argument replacing completely the previous backups.
	 * @param rUsers the list of registered users to backup. 
	 * @param mMss the list of managed message servers to backup.
	 * @throws IOException if an Input/Output problem occurred.
	 */
	public void backupWholeData(List<RegisteredUser> rUsers, List<ManagedMessageServer> mMss) throws IOException;

	/**
	 * @param restoredUsers a list where the restored users data must be copied to. 
	 * @param restoredMss a list where the restored servers data must be copied to.
	 * @return the number of read datas (message servers + users).
	 * @throws IOException if an Input/Output problem occurred.
	 * @throws ClassNotFoundException if a version compatibility problem occurred.
	 * @throws InvalidClassException if a version compatibility problem occurred.
	 * @throws StreamCorruptedException if the input stream could not be read.
	 * @throws OptionalDataException if the backupped datas were not expected.
	 * @throws IllegalArgumentException if the lists given by argument are null or not empty.
	 */
	public int restoreData(List<RegisteredUser> restoredUsers, List<ManagedMessageServer> restoredMss) throws IOException, ClassNotFoundException, InvalidClassException, StreamCorruptedException, OptionalDataException, IllegalArgumentException;
	
	/** Backup a single user data.
	 * @param reg the user whose data must be backupped.
	 * @throws IOException if an error occured while writing.
	 */
	public void backupUser(RegisteredUser reg) throws IOException, FileNotFoundException;
	
	/** Backup a single server data.
	 * @param ms the server whose data must be backupped.
	 * @throws IOException if an error occured while writing.
	 */
	public void backupMs(ManagedMessageServer ms) throws IOException;
	
	/** Remove a user contained in the backup. 
	 * @param reg the user to remove.
	 * @throws IOException if an InputOutput exception occurred.
	 */
	public void removeBackuppedUser(RegisteredUser reg) throws IOException;

	/** Remove a server contained in the backup. 
	 * @param ms the server to remove.
	 * @throws IOException if an InputOutput exception occurred.
	 */
	public void removeBackuppedMs(ManagedMessageServer ms) throws IOException;
	
}
