//
//
//  Generated by StarUML(tm) Java Add-In
//
//  @ Project : Untitled
//  @ File Name : BootstrapCore.java
//  @ Date : 16/04/2007
//  @ Author : 
//
//



package bootstrapP;

import java.io.IOException;
import java.io.InvalidClassException;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.rmi.AlreadyBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Date;
import java.util.List;
import java.util.Vector;

/** The main class of the Bootstrap Server, witch starts the services.
 * @author Marco Cornolti
 *
 */
public class BootstrapCore {
	private static Date upTime = new Date();
	protected static BackupperInterface mainBackupper;
	
	/**The main method to launch the Bootstrap Server, activating all the threads, publishing the rmi services, and restoring the data from the filename previousely used as backup.
	 * @param args the arguments.
	 */
	public static void main(String[] args) {

        int rmiPort=-1;
        int udpPort=-1;
        String backupFile=null;
        
        if (args.length<2){
            System.out.println("Gossip Bootstrap launching syntax:\n gossip rmi_port udp_port [backup_file]");
            System.exit(-1);
        }
        if (args.length>=3) backupFile = args[2];
        try{
            rmiPort=Integer.parseInt(args[0]);
            udpPort=Integer.parseInt(args[1]);
        } catch (NumberFormatException e){
            System.out.println("Could not parse a port number. Strings ["+args[0]+"] and ["+args[1]+"] are not valid.");
            System.exit(-1);
        }
        
        
        System.out.println("Loading GOSSIP Bootstrap Server...");
        
        if (backupFile==null)
        	System.out.println("WARNING: the Bootstrap Server is NOT going to save permanently any data, so every time it gets restarted all the users and servers data will be lost. To save the data, restart the server specifying a file to save the data to.");
        else{
        	try {
				System.out.println("Using file "+backupFile+" as backup.");
                mainBackupper = new FileBackupper(backupFile);
                List<RegisteredUser> restoredUsers = new Vector<RegisteredUser>();
                List<ManagedMessageServer> restoredMss= new Vector<ManagedMessageServer>();
                int restoredDataN = mainBackupper.restoreData(restoredUsers, restoredMss);
                MessageServersDatabase.restoreData(restoredMss);
                UsersDatabase.restoreData(restoredUsers);
			    if ( restoredDataN != 0)
                	System.out.println("Restoring of "+restoredDataN+" backupped records done.");
                else {
                	System.out.println("Backup file is empty and will be used to store new data.");
                }
        	} catch (InvalidClassException e) {
        		System.out.println("Could not read data from input file. Saved datas version and current Bootstrap version may differ.");
        		System.exit(-1);
        	} catch (StreamCorruptedException e) {
        		System.out.println("Could not read data from input file. The input stream is corrupted.");
        		System.exit(-1);
        	} catch (OptionalDataException e) {
           		System.out.println("Could not read data from input file. Unexpected data in file.");
        		System.exit(-1);
 			} catch (IllegalArgumentException e) {
 				System.out.println("Could not read data from input file. Internal error.");
        		System.exit(-1);
        	} catch (IOException e) {
        		System.out.println("Could not read data from input file. I/O error.");
        		System.exit(-1);
        	} catch (ClassNotFoundException e) {
        		System.out.println("Could not read data from input file. Saved datas version and current Bootstrap version may differ.");
        		System.exit(-1);
        	}
        }
        
        try{
            System.out.print("Creating local registry at localhost:"+rmiPort+"... ");
            Registry MSreg = LocateRegistry.createRegistry(rmiPort);
            MSreg.bind("gossipBS", new Communication());
            System.out.println("Done.");
        }
        catch (RemoteException e){
            System.out.println("Failed. Could not create RMI Registry.");
            System.exit(-1);
        } catch (AlreadyBoundException e){
            System.out.println("Failed. The service is already started.");
            System.exit(-1);
        }
        
        System.out.print("Starting load listener thread on UDP port "+udpPort+"... ");
        Thread loadThread = new LoadListener(udpPort);
        loadThread.start();
        System.out.println("Done.");
        
        System.out.print("Starting human interface... ");
        Thread humThread = new HumanListener();
        humThread.start();
        System.out.println("Done.");
	
	}
	
	/**Get the the time the server has started by.
	 * @return the time in milliseconds the server has started by.
	 */
	public static long getUpTime(){
		return (new Date()).getTime() - upTime.getTime();
	}
}
