//
//
//  Generated by StarUML(tm) Java Add-In
//
//  @ Project : Untitled
//  @ File Name : MessageServersDatabase.java
//  @ Date : 16/04/2007
//  @ Author : 
//
//



package bootstrapP;

import java.net.*;
import java.rmi.RemoteException;
import java.util.*;

import common.interfaces.exceptions.WrongPasswordException;


/**A database containing all the Message servers managed by this BS and part of this Gossip system. 
 * @author Marco Cornolti
 */
public class MessageServersDatabase {
    private static List<ManagedMessageServer> vector = new Vector<ManagedMessageServer>();
    
	/**Adds a Message Server to the list of managed servers.
	 * @param ip the MS IP address.
	 * @param RMIPort the MS RMI port.
	 * @param UDPPort the MS UDP port.
	 * @param pw the MS password.
	 * @throws RemoteException if the message server could not be reached.
	 */
	public static synchronized void addMessageServer(InetAddress ip, int RMIPort, int UDPPort, String pw) throws RemoteException{
	    ManagedMessageServer ms = new ManagedMessageServer(ip,RMIPort, UDPPort,pw);
	    if (vector.contains(ms)) {
	    	HumanListener.currentInt.showMSAlreadyInseted();
	    }
	    if (!ms.safeInt.testPassword(pw)){
	    	HumanListener.currentInt.showWrongMSPassword(ip.getCanonicalHostName(),RMIPort,UDPPort);
	    	return;
	    }
		vector.add(ms);
	    Collections.sort(vector);
	    try{
	    	BootstrapCore.mainBackupper.backupMs(ms);
	    } catch (Exception e){
	    	HumanListener.currentInt.showCouldNotBackupMS(ip.getCanonicalHostName(), RMIPort, UDPPort);
	    }
	    HumanListener.currentInt.showMSAdded(ip.getCanonicalHostName(),RMIPort,UDPPort);
	}
	
	/**
	 * Reorders the Message Server by their load. 
	 */
	public static synchronized void reorder(){
		Collections.sort(vector);
	}
	
	/**Returns a MS from the Managed message servers list. 
	 * @param id the MS id
	 * @return The message server of index id or null if the id is not valid.
	 */
	public static synchronized ManagedMessageServer getMS(int id){
		if (vector.size()-1<id||id<0) return null;
		return vector.get(id);
	}
	
	/**Returns a MS with the IP address and UDP port passed by arguments 
	 * from the Managed message servers list. 
	 * @param ip the MS ip address
	 * @param UDPPort the MS UDP port
	 * @return the message server with the specified IP address and UDP port, or null if it
	 * does not exist.
	 */
	public static synchronized ManagedMessageServer getMS(InetAddress ip, int UDPPort){
		for (int i=0;i<vector.size();i++){
			ManagedMessageServer msi = vector.get(i);
			if (msi.getUDPPort()==UDPPort && msi.getIP().equals(ip))
				return msi;
		}
		return null;
	}
	
	/**Removes a message server from the managed message servers list. This action can only be done if the message server has no users assigned.
	 * @param id the ms id.
	 * @return true if the message server has been removed, false otherwise.
	 */
	public static synchronized boolean removeMS(int id){
		if (vector.size()-1<id||id<0) return false;
		ManagedMessageServer ms =  vector.get(id);
		try{
			if (!ms.safeInt.getUsersList(ms.getPw()).isEmpty()) return false;
		} catch (RemoteException e){
			return false;
		} catch (WrongPasswordException e) {
			HumanListener.currentInt.showWrongMSPassword(id);
			return false;
		}
			
		vector.remove(id);
		Collections.sort(vector);
		try{
			BootstrapCore.mainBackupper.removeBackuppedMs(ms);
		} catch (Exception e){
		  	HumanListener.currentInt.showCouldNotRemoveBackuppedMS(ms.getIP().getCanonicalHostName(), ms.getRMIPort(), ms.getUDPPort());
		}
		return true;
	}

	/**
	 * @return the number of managed message servers.
	 */
	public static int getMSNumber() {
		return vector.size();
	}

	/**
	 * @return the message servers list. This list and all its elements must not be modified and can be used just for reading (e.g. for backupping).
	 */
	public static synchronized List<ManagedMessageServer> getAllMS() {
		return vector;
	}

	/**Searches the list for a specified MS and returns its id. Takes O(logN) where N is the number of managed message servers.
	 * @param managerMS the ms to search for.
	 * @return the message server id or <0 if the server is not contained.
	 */
	public static synchronized int getMS(ManagedMessageServer managerMS) {
		return Collections.binarySearch(vector, managerMS);
	}
	
	/**Restores the list replacing it with the one given as arguments. Should only be used at boot time!
	 * @param mmss the message servers list.
	 */
	public static synchronized void restoreData(List<ManagedMessageServer> mmss){
		vector=mmss;
	}

	/**
	 * @return an iterator over the elements of the managed message servers list. This elements must nod be modified.
	 */
	public static synchronized Iterator<ManagedMessageServer> getMSIterator() {
		return vector.iterator();
	}

}
