package bootstrapP;

import java.io.Serializable;
import java.net.InetAddress;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.util.Vector;

import common.*;
import common.interfaces.MSForBSInterface;
import common.interfaces.exceptions.*;

/**A Safe Message Server to Bootstrap interface.<br>
 * Normally, when a Bootstrap losts its RMI reference to the Message Server, for example for a MS shutdown, any call to the message server interface throws a RemoteException, even if the server got back on-line.<br>
 * Using this class, any method calls the interface method, but when the interface is down, it tries to bring it up once again. 
 * @author Marco Cornolti
 *
 */
public class SafeMSForBSInterface implements common.interfaces.MSForBSInterface, Serializable{
	private static final long serialVersionUID = 1L;
	private common.interfaces.MSForBSInterface interf;
	int RMIPort;
	InetAddress ip;
	
	/**Safe MS for BS Interface constructor.
	 * @param ip the MS ip address.
	 * @param RMIPort the MS RMI port.
	 * @throws RemoteException if the server could not be reached.
	 */
	public SafeMSForBSInterface(InetAddress ip, int RMIPort) throws RemoteException{
		this.RMIPort = RMIPort;
		this.ip=ip;
	 	refreshInterface();
	}
	
	public void assign(String MSPw, User user, String userPw) throws RemoteException, BlockedException, WrongPasswordException {
		try{
			interf.assign(MSPw, user, userPw);
		} catch (RemoteException e){
			refreshInterface();
			interf.assign(MSPw, user, userPw);
		}
	}

	public void blockAssignation(String MSPw, boolean status) throws RemoteException, WrongPasswordException {
		try{
			interf.blockAssignation(MSPw, status);
		} catch (RemoteException e){
			refreshInterface();
			interf.blockAssignation(MSPw, status);
		}
	}

	public void deleteUserAdmin(User nick, String MSpw) throws RemoteException, WrongPasswordException, BlockedException, UserNotManagedException {
		try{
			interf.deleteUserAdmin(nick, MSpw);
		} catch (RemoteException e){
			refreshInterface();
			interf.deleteUserAdmin(nick, MSpw);
		}		
	}

	public Statistics getStatistics(String globalpw) throws RemoteException, WrongPasswordException {
		Statistics stats=null;
		try{
			stats = interf.getStatistics(globalpw);
		} catch (RemoteException e){
			refreshInterface();
			stats = interf.getStatistics(globalpw);
		}
		return stats;
	}

	public Vector<User> getUsersList(String globalpw) throws RemoteException, WrongPasswordException {
		Vector<User> users=null;
		try{
			users = interf.getUsersList(globalpw);
		} catch (RemoteException e){
			refreshInterface();
			users = interf.getUsersList(globalpw);
		}
		return users;
	}

	public void moveUser(String MSPw, User user, MessageServer ms) throws RemoteException, WrongPasswordException, UserNotManagedException, AssignationNotAcceptedException, GenericErrorException {
		try{
			interf.moveUser(MSPw, user, ms);
		} catch (RemoteException e){
			refreshInterface();
			interf.moveUser(MSPw, user, ms);
		}		
	}

	public void shutdown(String MSPw) throws RemoteException, WrongPasswordException, UsersStillPresentException, BlockedException {
		try{
			interf.shutdown(MSPw);
		} catch (RemoteException e){
			refreshInterface();
			interf.shutdown(MSPw);
		}		
	}

	public boolean testPassword(String pw) throws RemoteException {
		boolean test=false;
		try{
			test= interf.testPassword(pw);
		} catch (RemoteException e){
			refreshInterface();
			test= interf.testPassword(pw);
		}
		return test;
	}

	public void testPresence() throws RemoteException {
		try{
			interf.testPresence();
		} catch (RemoteException e){
			refreshInterface();
			interf.testPresence();
		}		
	}
	
	/**Refreshes the interface.
	 * @throws RemoteException if the interface could not be refreshed (the server is still down).
	 */
	private void refreshInterface() throws RemoteException{
		try{
			this.interf = (MSForBSInterface) Naming.lookup("rmi://"+ip.getHostAddress()+":"+RMIPort+"/gossipMS");
		} catch(Exception e){
			throw new RemoteException();
		}
	}

}
