//
//
//  Generated by StarUML(tm) Java Add-In
//
//  @ Project : Untitled
//  @ File Name : Statistics.java
//  @ Date : 19/03/2007
//  @ Author : 
//
//



package common;

import java.io.Serializable;
import java.util.Date;
import java.util.Vector;


/**This class represents a Statistics object, witch contains statistical data about a Message server, and
 * the static methods and datas to create these object. In particular, it contains a vector that contains
 * the times the messages has been sended at.<br>
 * Some methods of this class must be called during the Message Server activity, to let the data be updated. 
 * @author Marco Cornolti and Alessandro Franchini
 */
public class Statistics implements Serializable{

	private static final long serialVersionUID = 1L;
	private static Vector<Date> forwardedMessagesDates = new Vector<Date>();
	private int cacheEntries;
	private int managedUsers;
	private int onlineManagedUsers;
	private int queuedMsgsN;
    private int forwardedMessagesMinute;
    private int forwardedMessagesHour;
    private int forwardedMessages24Hours;
    private int forwardedMessages48Hours;
    private int forwardedMessages72Hours;
    private int forwardedMessages96Hours;
    private int forwardedMessages120Hours;
    private int forwardedMessages144Hours;
    
    private long freeMem;
    
    /**@param cacheEntries the entries in the MS cache (user-MS corrispondences)
     * @param managedUsers the number of managed users.
     * @param onlineManagedUsers the number of online managed users.
     * @param queuedMsgsN the number of messages contained in the users' queues.
     */
    public Statistics(int cacheEntries,int managedUsers, int onlineManagedUsers, int queuedMsgsN){
        this.cacheEntries=cacheEntries;
        this.managedUsers=managedUsers;
        this.onlineManagedUsers=managedUsers;
        this.queuedMsgsN=queuedMsgsN;
        this.forwardedMessagesMinute = forwardedMessagesSince(1);
        this.forwardedMessagesHour = forwardedMessagesSince(60);
        this.forwardedMessages24Hours = forwardedMessagesSince(60*24);
        this.forwardedMessages48Hours = forwardedMessagesSince(60*48);
        this.forwardedMessages72Hours = forwardedMessagesSince(60*72);
        this.forwardedMessages96Hours = forwardedMessagesSince(60*96);
        this.forwardedMessages120Hours = forwardedMessagesSince(60*120);
        this.forwardedMessages144Hours = forwardedMessagesSince(60*144);
        this.freeMem = Runtime.getRuntime().freeMemory();
    }
    
    /**Returns the number of messages forwarded since min minutes ago. Takes O(logN)
     * where N is the number of messages forwarding dates in the vector.
     * @param min the minutes of the mesaurement.
     * @return the number of messages forwarded in the last min minutes.
     */
    private static int forwardedMessagesSince(int min){
    	return fwMessSinceRec(new Date(new Date().getTime()-(1000*60*min)),0, forwardedMessagesDates.size()-1);
    }
    
    /**Returns the number of dates present in the forwardedMessagesDates after the date
     * given by argument. The range of positions the algorithm has to search must be specified.
     * The vector must be ordered (most recent date in the last position).
     * This method takes O(logN) where N is the number of dates in the vector.
     * @param from the date to start counting by.
     * @param start the first index of the searching range.
     * @param end the last index of the searching range. 
     * @return the number of messages forwarded from the date from, witch is included in the range start-end.
     */
    private static int fwMessSinceRec(Date from, int start, int end){
    	int last = forwardedMessagesDates.size()-1;
    	if (start>end) return 0;
    	if (start==end) {
    		if (forwardedMessagesDates.get(start).before(from))
    			return last-start;
    		else
    			return last-start+1;
    	}
    	if (forwardedMessagesDates.get(end).before(from)) return last-end;
    	if (forwardedMessagesDates.get(start).after(from)) return last+1;
    	
    	int pivotInd=(end+start)/2;
    	Date pivotDate=forwardedMessagesDates.get(pivotInd);
    	if (pivotDate.before(from))
    		return fwMessSinceRec(from, pivotInd+1, end);
    	else
    		return fwMessSinceRec(from, start, pivotInd); 
    }
    
    /**
     * @return the number of cache entries contained in this statistics.
     */
    public int getCacheEntries() {
        return cacheEntries;
    }
    
    /**
     * @return the amount of free memory.
     */
    public long getFreeMem() {
        return freeMem;
    }
	
	/**
	 * @return the number of managed users.
	 */
	public int getManagedUsers() {
        return managedUsers;
	}
	
	/**
	 * @return the number of online managed users.
	 */
	public int getOnlineManagedUsers() {
	    return onlineManagedUsers;
	}
	
	/**
	 * @return the number of queued messages.
	 */
	public int getQueuedMessageNumber() {
	    return queuedMsgsN;
	}
	
	/**
	 * @return the amount of messages forwarded in the last minute.
	 */
	public int getForwardedMessagesMinute() {
	    return forwardedMessagesMinute;
	}

	/**
	 * @return the amount of messages forwarded in the last hour.
	 */
	public int getForwardedMessagesHour() {
	    return forwardedMessagesHour;
	}
    
	/**
	 * @return the amount of messages forwarded in the last day.
	 */
	public int getForwardedMessages24Hours() {
	    return forwardedMessages24Hours;
	}

	/**
	 * @return the amount of messages forwarded in the last 2 days.
	 */
	public int getForwardedMessages48Hours() {
	    return forwardedMessages48Hours;
	}
	
	/**
	 * @return the amount of messages forwarded in the last 3 days.
	 */
	public int getForwardedMessages72Hours() {
	    return forwardedMessages72Hours;
	}
	
	/**
	 * @return the amount of messages forwarded in the last 4 days.
	 */
	public int getForwardedMessages96Hours() {
	    return forwardedMessages96Hours;
	}
	
	/**
	 * @return the amount of messages forwarded in the last 5 days.
	 */
	public int getForwardedMessages120Hours() {
	    return forwardedMessages120Hours;
	}
	
	/**
	 * @return the amount of messages forwarded in the last 6 days.
	 */
	public int getForwardedMessages144Hours() {
	    return forwardedMessages144Hours;
	}
	
	/**Must be called whenever a new message is forwarded. Adds the current time to the vector.
	 */
	public static void increaseForwardedMessages() {
		int i=0;
		while (i<=forwardedMessagesDates.size()-1){
			Date iDate = forwardedMessagesDates.get(i);
			if (iDate.before(new Date(new Date().getTime()-144*60*60*1000))) forwardedMessagesDates.remove(i);
			else break;
		}
			
		forwardedMessagesDates.add(new Date());
	}

	/**
	 * @return the amount of messages forwarded in the last 24 hours.
	 */
	public static int get24HourFwMsgs() {
		return forwardedMessagesSince(60*24);
	}
}
