//
//
//  Generated by StarUML(tm) Java Add-In
//
//  @ Project : Untitled
//  @ File Name : MSForServletInterface.java
//  @ Date : 19/03/2007
//  @ Author : 
//
//



package common.interfaces;

import java.net.InetAddress;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Vector;

import common.interfaces.exceptions.*;


/**This class represents the methods that the object exported by the Gossip Message Server must implement to let a Gossip Servlet use it.
 * @author Marco Cornolti and Alessandro Franchini
 */
public interface MSForServletInterface extends Remote{
	
	/**Used by a servlet to tell the message server to add another user in his contact list as friend. 
	 * @param user the invoking user.
	 * @param pw the invoking user password.
	 * @param userToAdd the user to add.
	 * @throws UserNotManagedException if the user is not managed by the local MS. 
	 * @throws UserBlockedException if the user is blocked.
	 * @throws WrongPasswordException if the password is not right. 
	 * @throws UserAlreadyInsertedException if the user is already contained in the incoming list.
	 * @throws RemoteException if a communication problem ocurred.
	 */
	public abstract void addToFriends(common.User user, String pw, common.User userToAdd) throws RemoteException, UserNotManagedException, WrongPasswordException, UserBlockedException, UserAlreadyInsertedException;
	
	
	/**Gets a User IP address and TCP port to directely connect to its servlet.
	 * @param user the user whose connection datas is wanted. Only the nickname is considered. 
	 * @return the user with the correctly obtained IP address and TCP port. 
	 * @throws RemoteException if a communication problem occurs.
	 * @throws UserNotExistsException if the specificed user nickname does not exists.
	 * @throws GenericErrorException if another error occurred (i.e. a communication problem with another message server).
	 * @throws UserBlockedException if the user is blocked.
	 */
	public abstract common.User getUserContact(common.User user) throws RemoteException, UserNotExistsException, GenericErrorException, UserBlockedException;
	
	/**Marks userToBlock in the user's contact list as blocked, so that no more messages from that user are accepted.
	 * @param user the user.
	 * @param pw the user's password.
	 * @param userToBlock the user to block.
	 * @throws UserNotManagedException if user is not managed by this message server. 
	 * @throws UserNotExistsException if the userToBlock is not present in the contact list.
	 * @throws WrongPasswordException if the user's password is wrong.
	 * @throws UserBlockedException if the user is blocked
	 */
	public abstract void blockContact(common.User user, String pw, common.User userToBlock) throws RemoteException, UserNotManagedException, UserNotExistsException, WrongPasswordException, UserBlockedException;

	/**Removes userToRemove from user's contact list, so that no more messages from that user are accepted.
	 * @param user the user.
	 * @param pw the user's password.
	 * @param userToRemove the user to remove.
	 * @throws UserNotManagedException if user is not managed by this message server. 
	 * @throws UserNotExistsException if the userToRemove is not present in the contact list.
	 * @throws WrongPasswordException if the user's password is wrong.
	 * @throws UserBlockedException if the user is currently blocked.
	 */
	public abstract void removeContact(common.User user, String pw, common.User userToRemove) throws RemoteException, UserNotManagedException, UserNotExistsException, WrongPasswordException, UserBlockedException;

    /**Downloads the messages stored in the Message server for a user. After calling this method, the
     * servlet has to call deleteDownloadedMsgs to free the server deleting old messages.
     * @param user the user whose messages are wanted.
     * @param pw the user's password
     * @return An empty Vector if the user has no messages. A vector containing the user's messages otherwise.
     * @throws RemoteException if a communication problem occurs.
     * @throws UserNotManagedException if the user is not managed by this message server.
     * @throws UserBlockedException if the user is blocked (i.e. to be moved).
     * @throws WrongPasswordException if the given password is wrong.
     */
    public abstract Vector<common.Message> downloadMsgs(common.User user, String pw) throws RemoteException, UserNotManagedException, UserBlockedException, WrongPasswordException;
    
    /**Deletes the messages that are already downloaded by a User.
     * @param user the user.
     * @param pw the user password.
     * @throws RemoteException if a communication problem occurs.
     * @throws UserNotManagedException if the user is not managed by the MS.
     * @throws WrongPasswordException if the password is wrong.
     * @throws UserBlockedException is the user is blocked.
     */
    public abstract void deleteDownloadedMsgs(common.User user, String pw) throws RemoteException, UserNotManagedException, WrongPasswordException, UserBlockedException;
    
    /**Sets a user state. Must be invoked whenever the user gets on-line or is about
     * to go off-line.
     * @param user the user
     * @param pw the user password
     * @param state true if the state is on-line, false if the state is off-line
     * @throws RemoteException if a communication problem occurs.
     * @throws UserNotManagedException if the user is not managed by the MS.
     * @throws WrongPasswordException if the password is wrong.
     * @throws GenericErrorException if another error occurred, i.e. the server could not define the servlet ip address.
     * @throws UserBlockedException if the user is blocked.
     */
    public abstract void setState(common.User user, String pw, boolean state) throws RemoteException, UserNotManagedException, WrongPasswordException, GenericErrorException, UserBlockedException;
    
	/**Forwards a message. The message recipient or the message sender must be managed
	 * by the MS. If the recipient is managed by the MS, the message gets stored and is ready to
	 * be downloaded. If the recipient is not local but the sender is, the MS forwards the message
	 * to the correct MS.
	 * @param msg the message to forward.
	 * @throws UserNotManagedException if the MS does not manage the recipient nor the sender. 
	 * @throws NotAllowedException if the recipient user does have the sender in its blockedlist. 
	 * @throws UserNotExistsException if the recipient does not exist in the whole system. 
	 * @throws GenericErrorException if another error has occurred. 
	 * @throws SystemErrorException if a system error occurred.
	 */
	public abstract void forwardMessage(common.Message msg) throws RemoteException, NotAllowedException, UserNotManagedException, GenericErrorException, UserNotExistsException, SystemErrorException;
	
	/**Returns user list of contacts.
	 * @param user the user
	 * @param pw the user password
	 * @return the contact list.
	 * @throws RemoteException if a communication problem occurred.
	 * @throws UserNotManagedException if the user is not managed by this message server.
	 * @throws WrongPasswordException if the user's password is wrong.
	 * @throws UserBlockedException if the user is blocked.
	 */
	public abstract common.ContactList getContactList(common.User user, String pw) throws RemoteException, UserNotManagedException, WrongPasswordException, UserBlockedException;
	
	/**Changes the user password.
	 * @param user the user
	 * @param oldPw the old password
	 * @param newPw the new password to set
	 * @throws RemoteException if a communication problem occurred.
	 * @throws UserNotManagedException if the user is not managed by the MS.
	 * @throws WrongPasswordException if the old password is wrong.
	 * @throws UserBlockedException if the user is blocked.
	 */
	public abstract void changePassword(common.User user, String oldPw, String newPw) throws RemoteException, UserNotManagedException, WrongPasswordException, UserBlockedException;
	/**Tests the servlet port trying to connect to them. 
	 * @param port the port to test.
	 * @param ip the servlet ip.
	 * @return true if the test was correctly terminated, false otherwise.
	 * @throws RemoteException if a communication problem occurred while invoking this method. 
	 */
	public abstract boolean testPorts(int port, InetAddress ip) throws RemoteException;
		
}
