package messageServerP;

import java.io.IOException;
import java.util.List;

/**An interface that represents a backupper for a Gossip Message Server. A backupper is a unit that can save and restore
 * any working data of the server, that is the managed users. 
 * @author Marco Cornolti
 *
 */
public interface BackupperInterface {
	/**Backups the whole data given as arguments.
	 * @param mUsers the managed users to backup list.
	 * @throws IOException if a problem occurred while writing the backup.
	 */
	public void backupWholeData(List<ManagedUser> mUsers) throws IOException;
	/**
	 * @param restoredUsers an empty list where restored users will be stored.
	 * @return the number of restored users.
	 * @throws IOException if a problem occurred while reading the backup.
	 * @throws ClassNotFoundException if the read object class is not recognized.
	 * @throws IllegalArgumentException if the list given as argument is not empty or is null.
	 */
	public int restoreData(List<ManagedUser> restoredUsers) throws IOException, ClassNotFoundException, IllegalArgumentException;
	/**Replaces the status of a user in the backup or adds the user if it is not backupped.
	 * @param newUserStatus the new user status.
	 * @throws IOException if a problem occurred while reading or writing the backup (broken data, incorrect version).
	 */
	public void updateUser(ManagedUser newUserStatus) throws IOException;
	/**Backups a new user adding it to the backup.
	 * @param usr the user to backup
	 * @throws IOException if the user addition failed (broken old backup, not enought space left, ...).
	 */
	public void backupUser(ManagedUser usr) throws IOException;
	/**Removes a user from the backup.
	 * @param usr the user to delete (only the nickname is considered).
	 * @throws IOException if the user deletion failed (user was not present, i/o error, ...)
	 */
	public void removeBackuppedUser(ManagedUser usr) throws IOException;
	
}
