package messageServerP;

import java.net.InetAddress;
import java.rmi.RemoteException;

import common.*;
import common.interfaces.MSForMSInterface;
import common.interfaces.exceptions.*;

/**A Safe Message server to Message server interface.
 * Normally, when a Message Server losts its RMI reference to another Message Server, for example for a MS shutdown, any call to the message server interface throws a RemoteException, even if the server got back on-line.
 * Using this class, any method calls the interface method, but when the interface is down, it tries to bring it up once again. 
 * @author Marco Cornolti
 *
 */
public class SafeMSForMSInterface implements MSForMSInterface{
	private InetAddress ip;
	private int port;
	MSForMSInterface interf;
	
	/**Safe MS for MS Interface constructor.
	 * @param ip the MS ip address.
	 * @param port the MS port.
	 */
	public SafeMSForMSInterface(InetAddress ip, int port) throws RemoteException{
		this.port=port;
		this.ip=ip;
		refreshInterface();
	}

	public void assignMovedUser(ManagedUser user) throws RemoteException, AssignationNotAcceptedException {
		try{
			interf.assignMovedUser(user);
		} catch (RemoteException e){
			refreshInterface();
			interf.assignMovedUser(user);
		}
	}

	public void forwardMessage(Message msg) throws RemoteException, NotAllowedException, UserNotManagedException, GenericErrorException, UserNotExistsException, SystemErrorException {
		try{
			interf.forwardMessage(msg);
		} catch (RemoteException e){
			refreshInterface();
			interf.forwardMessage(msg);
		}
	}

	public User getManagedUserContact(User user) throws RemoteException, UserNotManagedException {
		User mu=null;
		try{
			mu= interf.getManagedUserContact(user);
		} catch (RemoteException e){
			refreshInterface();
			mu= interf.getManagedUserContact(user);
		}	
		return mu;
	}

	public boolean inListOfUserContains(User listUser, User user) throws RemoteException, UserNotManagedException {
		boolean cont=false;
		try{
			cont= interf.inListOfUserContains(listUser, user);
		} catch (RemoteException e){
			refreshInterface();
			cont= interf.inListOfUserContains(listUser, user);
		}	
		return cont;
	}

	/**Refreshes the interface.
	 * @throws RemoteException if the interface could not be refreshed.
	 */
	private void refreshInterface() throws RemoteException{
		try{
			this.interf = (MSForMSInterface)java.rmi.Naming.lookup("rmi://"+ip.getHostAddress()+":"+port+"/gossipMS");
		}catch (Exception e){
			throw new RemoteException();
		}
	}
}
