package servlet;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.DataOutputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import common.Message;
import common.User;
import common.interfaces.exceptions.UserAlreadyInsertedException;

/**
 * This graphic interface shows to the user when another user contacts him the first time.
 * @author Alessandro Franchini
 *
 */
public class ConfirmInterface extends JFrame {
	
	private static final long serialVersionUID = 1L;
	final String yes = "Yes";
	final String no = "No";
	
	/**
	 * The constructor
	 * @param name the user
	 * @param so the socket
	 */
	public ConfirmInterface(final User name, final Socket so){
		JPanel panel = new JPanel();
		setSize(300,100);
		setTitle("GOSSIP");
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		JRadioButton[] radio = new JRadioButton[2];
		final ButtonGroup group = new ButtonGroup();
		JButton confirm;
		String request = "The user " + name.getNick() + " ask to you to become his friend:";
		radio[0] = new JRadioButton("Yes");
		radio[0].setActionCommand(yes);
		radio[1] = new JRadioButton("No");
		radio[1].setActionCommand(no);
		group.add(radio[0]);
		group.add(radio[1]);
		radio[0].setSelected(true);
		confirm = new JButton("Confirm");
		confirm.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				String command = group.getSelection().getActionCommand();
				User temp = AccessOperation.getUser(name);
				if(so==null){ 
					boolean flag = confirm(temp, command);
					if(flag){
						try {
							Message msg = new Message("Type::Check",name, ClientCore.myself);
							Socket so = new Socket(temp.getIP(),temp.getPort());
							OutputStream os = so.getOutputStream();
							ObjectOutputStream dos = new ObjectOutputStream(os);
							dos.writeObject(msg);
							so.close();
						} catch (Exception e) {
							System.out.println("The user is offline");
						}
					}
					dispose();
				}
				else{
					OutputStream os;
					try {
						os = so.getOutputStream();
						DataOutputStream dos = new DataOutputStream(os);
						boolean check = confirm(name ,command);
						if(check)
							dos.writeBoolean(true);
						else
							dos.writeBoolean(false);
						so.close();
						dispose();
					} catch (Exception e) {
						System.out.println("Utente scollegato");
					}
				}
				
			}
			
		});
		JLabel form = new JLabel(request);
		panel.add(form);
		panel.add(radio[0]);
		panel.add(radio[1]);
		panel.add(confirm);
		getContentPane().add(panel);	
	}
	
	/**
	 * This method, according to the user's choice, locks or adds to friends the other user.
	 * @param name the user
	 * @param command the user's choice
	 * @return true if the choice is yes; else false.
	 */
	boolean confirm(User name, String command){
		boolean flag = false;
		User temp = AccessOperation.getUser(name);
		if(command.equals(yes)){
			try {
				temp.setOnline(true);
				temp.setFriend(true);
				GossipInterface.utenti.addUser(temp,true);
				GossipInterface.inLinea.addElement(temp.getNick());
				GossipInterface.onLine.repaint();
				AccessOperation.changeStateUser(0,temp);
				flag = true;
			} catch (UserAlreadyInsertedException e) {
				System.out.println("The user is already inserted");
			}
		}
		else{
			temp.setOnline(true);
			temp.setFriend(false);
			try {
				GossipInterface.utenti.addUser(temp,false);
				GossipInterface.neri.addElement(temp.getNick());
			} catch (UserAlreadyInsertedException e) {
				System.out.println("The user is already inserted");
			}
			AccessOperation.changeStateUser(1,temp);
		}
		return flag;
		
	}
}
