package servlet;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.DataInputStream;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;

import common.Message;
import common.User;
import common.interfaces.exceptions.UserAlreadyInsertedException;

/**
 * This graphic interface is of use to change the friendship's status with
 * another user and to add another user.
 * @author Alessandro Franchini
 *
 */
public class ContattInterface extends JFrame {

	private static final long serialVersionUID = 1L;
	final JPanel contatti;
	JButton aggiungiIn = new JButton("Sblocca contatto");
	JButton aggiungiOut = new JButton("Blocca contatto");
	JButton add = new JButton("Aggiungi nuovo utente");
	JList entrata, uscita;
	JTextField contact;
	public DefaultListModel amici,nemici;
	
	/**
	 * The constructor
	 *
	 */
	public ContattInterface(){
		setTitle("Contatti");
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setSize(600,500);
		contatti = new JPanel();
		contatti.setLayout(null);
		amici = new DefaultListModel();
		nemici = new DefaultListModel();
		setList(amici,nemici);
		entrata = new JList(amici);
		JScrollPane scrollin = new JScrollPane(entrata);
		scrollin.setBounds(10,10,270,300);
		contatti.add(scrollin);
		aggiungiOut.setBounds(40,320,200,20);
		contatti.add(aggiungiOut);
		uscita = new JList(nemici);
		JScrollPane scrollout = new JScrollPane(uscita);
		scrollout.setBounds(300,10,270,300);
		contatti.add(scrollout);
		aggiungiIn.setBounds(330,320,200,20);
		contatti.add(aggiungiIn);
		contatti.add(new JSeparator(SwingConstants.HORIZONTAL));
		contact = new JTextField();
		contact.setBounds(10,400,270,25);
		contatti.add(contact);
		add.setBounds(300,400,200,20);
		contatti.add(add);
		setupEventi();
		entrata.setSelectedIndex(0);
		entrata.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		uscita.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		entrata.setVisibleRowCount(-1);
		uscita.setVisibleRowCount(-1);
		uscita.setSelectedIndex(0);
		if(amici.size()==0){
			aggiungiOut.setEnabled(false);			
		}
		if(nemici.size()==0){
			aggiungiIn.setEnabled(false);
		}
		getContentPane().add(contatti);
		
	}
	
	/**
	 * This method manages the events joined to the graphic interface.
	 *
	 */
	public void setupEventi(){
		MouseListener mouseIn = new MouseAdapter(){
			public void mouseClicked(MouseEvent e) {
		         if (e.getClickCount() == 2) {
		             change(0);  
		          }
		     }

		};
		entrata.addMouseListener(mouseIn);
		MouseListener mouseOut = new MouseAdapter(){
			public void mouseClicked(MouseEvent e) {
		         if (e.getClickCount() == 2) {
		             change(1);  
		          }
		     }

		};
		uscita.addMouseListener(mouseOut); 
		aggiungiOut.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				change(0);
			}
		});
		aggiungiIn.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				change(1);	
			}			
		});
		add.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {
				if(contact.getText().length()==0){
					JOptionPane.showMessageDialog(getContentPane(), "Insert the user name", "Gossip", JOptionPane.ERROR_MESSAGE);
					return;
				}
				User temp = new User(contact.getText(),null,0,false,true);
				temp = AccessOperation.getUser(temp);
				contact.setText("");
				if(temp==null){
					return;
				}
				
				boolean flag = AccessOperation.contactUser(temp);
				if(flag){
					temp.setOnline(true);
					temp.setFriend(true);
					GossipInterface.repaintWindow(temp,1,0);
				}
				else{
					temp.setOnline(false);
					temp.setFriend(true);
					GossipInterface.repaintWindow(temp,0,0);
				}
				try {
					GossipInterface.utenti.addUser(temp,true);
					AccessOperation.changeStateUser(0,temp);
					amici.addElement(temp.getNick());
					entrata.repaint();
					uscita.repaint();
					aggiungiOut.setEnabled(true);
				} catch (UserAlreadyInsertedException e) {
					System.out.println("The user is already inserted");
				}					
			}
			
		});
	}
	
	
	/**
	 * This method manages the moving of a user.
	 * @param form 0 to lock the user; 1 to unlock the user.
	 */
	void change(int form){
		if (form==0){
			String temp = (String) entrata.getSelectedValue();
			GossipInterface.neri.addElement(temp);
			amici.removeElement(temp);
			nemici.addElement(temp);
			User user = (User) GossipInterface.utenti.getUser(temp);
			GossipInterface.utenti.setFriend(user,false);
			AccessOperation.changeStateUser(1,user);
			Socket so = null;
			Message msg = new Message("Type::Offline",user,ClientCore.myself);
			OutputStream os;
			try {
				so = new Socket(user.getIP(),user.getPort());
				os = so.getOutputStream();
				ObjectOutputStream oos = new ObjectOutputStream(os);
				oos.writeObject(msg);
			} catch (Exception e) {
				System.out.println("The user is offline");
			}
			if(user.isOnline())
				GossipInterface.repaintWindow(user,1,1);
			else
				GossipInterface.repaintWindow(user,0,1);
			
				
			entrata.repaint();
			uscita.repaint();
			entrata.setSelectedIndex(0);
			uscita.setSelectedIndex(0);
			if(amici.size()==0){
				aggiungiOut.setEnabled(false);			
			}
			aggiungiIn.setEnabled(true);
		}
		else{
			
			String temp = (String) uscita.getSelectedValue();
			GossipInterface.neri.removeElement(temp);
			User user = (User) GossipInterface.utenti.getUser(temp);
			GossipInterface.utenti.setFriend(user,true);
			AccessOperation.changeStateUser(0,user);
			user = AccessOperation.getUser(user);
			Socket so = null;
			try {
				Message msg = new Message("Type::Check",user, ClientCore.myself);
				so = new Socket(user.getIP(),user.getPort());
				OutputStream os = so.getOutputStream();
				ObjectOutputStream oos = new ObjectOutputStream(os);
				oos.writeObject(msg);
				InputStream is = so.getInputStream();
				DataInputStream dis = new DataInputStream(is);
				GossipInterface.utenti.getUser(user.getNick()).setOnline(dis.readBoolean());
				so.close();
			} catch (Exception e) {
				GossipInterface.utenti.setOnLine(user,false);
			}
			if(GossipInterface.utenti.getUser(user.getNick()).isOnline()){
					GossipInterface.repaintWindow(user,1,0);
			}
			else
				GossipInterface.repaintWindow(user,0,0);
			amici.addElement(temp);
			nemici.removeElement(temp);
			entrata.repaint();
			uscita.repaint();
			entrata.setSelectedIndex(0);
			uscita.setSelectedIndex(0);
			aggiungiOut.setEnabled(true);			
			if(nemici.size()==0)
				aggiungiIn.setEnabled(false);
			
		}
	}
	
	/**
	 * This method initializes the lists of the graphic interface
	 * @param listAmici the friends list
	 * @param listaNeri the lock list
	 */
	public void setList(DefaultListModel listAmici, DefaultListModel listaNeri){
			for(int i = 0; i<GossipInterface.utenti.getSize();i++){
				User temp = (User) GossipInterface.utenti.getUser(i);
				if(temp.isFriend())
					listAmici.addElement(temp.getNick());
				else
					listaNeri.addElement(temp.getNick());
			}
		}
}
