package servlet;
import java.io.DataInputStream;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;

import common.ContactList;
import common.Message;
import common.User;
import common.interfaces.exceptions.UserAlreadyInsertedException;

/**
 * This class checks if the users of the contacts list are on-line.
 * @author Alessandro Franchini
 *
 */
public class ControlStatus {
	private ContactList temp,list;
	private User user;
	private int finalsize;
	
	/**
	 * The constructor
	 * @param cl the contacts list
	 * @param user the user
	 */
	public ControlStatus(ContactList cl, User user){
		temp = cl;
		list = new ContactList();
		this.user = user;
		finalsize = temp.getSize();
	}
	
	/**
	 * This method gets and removes the first user of the list.
	 * @return the first user of the list.
	 */
	public synchronized User getUser(){
		User user = temp.getUser(0);
		temp.removeUser(user);
		return user;	
	}
	
	/**
	 * This method adds a user of the new list.
	 * @param name
	 */
	private synchronized void addUser(User name){
		try {
			list.addUser(name, name.isFriend());
		} catch (UserAlreadyInsertedException e) {
			
		}
	}
	
	/**
	 * If the user is a friend, checks if he is online. After the check, adds it at the
	 * new contacts list.
	 * @param name the user
	 */ 
	public void setUser(User name){
		if(!name.isFriend()){
			addUser(name);
			return;
		}
		User temp = AccessOperation.getUser(name);
		Socket so = null;
		try {
			Message msg = new Message("Type::Check",temp, user);
			so = new Socket(temp.getIP(),temp.getPort());
			OutputStream os = so.getOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(os);
			oos.writeObject(msg);
			InputStream is = so.getInputStream();
			DataInputStream dis = new DataInputStream(is);
			temp.setOnline(dis.readBoolean());
			so.close();
			addUser(temp);
		} catch (Exception e) {
			temp.setOnline(false);
			addUser(temp);
		}
	}
	
	/**
	 * Returns the final contacts list.
	 * @return the contacts list.
	 */
	public ContactList getFinalList(){
		while(list.getSize()<finalsize);
		return list;
	}
}

/**
 * This class effects the check for a user.
 * @author Alessandro Franchini
 *
 */
class threadStatus extends Thread{
	ControlStatus cs;
	
	/**
	 * The constructor
	 * @param cs the control status
	 */
	public threadStatus(ControlStatus cs){
		this.cs = cs;
	}
	
	public void run(){
		User temp = cs.getUser();
		cs.setUser(temp);
		return;
	}
}
