package testing;

import java.rmi.Naming;
import java.util.Vector;

import common.ContactList;
import common.Message;
import common.MessageServer;
import common.User;
import common.interfaces.BSForServletInterface;
import common.interfaces.MSForServletInterface;
import common.interfaces.exceptions.*;

public class Servlet40Messages {

	public static void main (String[] argv){
		BSForServletInterface bsInt = null;
		try {
			//test da eseguire dopo i test 10, 25, 30
			
			//Prove di un solo utente su un solo MS...
			bsInt = (BSForServletInterface) Naming.lookup("rmi://localhost:1111/gossipBS");
			User myself = new User("marco",null,0,true,true);
			MSForServletInterface msi=null;
			MessageServer ms = bsInt.getMS(myself);
			//System.out.println("linking to: rmi://"+ms.getIP()+":"+ms.getPort()+"/gossipMS");
			msi = (MSForServletInterface) Naming.lookup("rmi://"+ms.getIP().getHostName()+":"+ms.getPort()+"/gossipMS");
			
			
			try{
				//prova: invio messaggio a me stesso (aggiunto come amico)
				msi.forwardMessage(new Message("il plesso solare e' importante\ntrattalo bene",myself,myself));
				System.out.println("Forward consentito ok.");
			} catch (NotAllowedException e){
				System.out.println("ERRORE Forward non consentito: "+e);
			}
			
			try{
				//prova: invio messaggio a utente inesistente
				msi.forwardMessage(new Message("i topi non avevano nipoti!",new User("uomoinvisibile",null,0,true,true),myself));
				System.out.println("ERRORE forward ad utente inesistente consentito.");
			} catch (UserNotExistsException e){
				System.out.println("Forward a utente inesistente non consentito OK: "+e);
			}

			User chuck = new User("chuck",null,0,true,true);
		    
			try{
				//prova: invio messaggio a Chuck, utente non amico sullo stesso server
				msi.forwardMessage(new Message("yabadabadooo!",chuck,myself));
				System.out.println("OK forward ad utente non amico consentito.");
			} catch (NotAllowedException e){
				System.out.println("ERRORE: Forward a utente non amico ma non bloccato non consentito: "+e);
			}

			
			printContactList(chuck.getNick(), msi.getContactList(chuck, "norris"));
			printContactList(myself.getNick(), msi.getContactList(myself, "pallone"));
			
			System.out.println("Chuck inserisce Marco nella lista degli amici");
			msi.addToFriends(chuck, "norris", myself);
			printContactList(chuck.getNick(), msi.getContactList(chuck, "norris"));
			printContactList(myself.getNick(), msi.getContactList(myself, "pallone"));
			
			System.out.println("Chuck inserisce Marco nella lista degli amici");
			try{
				msi.addToFriends(chuck, "norris", myself);
				System.out.println("ERRORE: inserimento di un utente gia' inserito permesso.");
			} catch (UserAlreadyInsertedException e){
				System.out.println("OK: inserimento di un utente gia' inserito non permesso.");
			}
			
			printContactList(chuck.getNick(), msi.getContactList(chuck, "norris"));
			printContactList(myself.getNick(), msi.getContactList(myself, "pallone"));
			
			System.out.println("Marco inserisce Chuck nella lista degli amici");
			msi.addToFriends(myself, "pallone", chuck);
			
			printContactList(chuck.getNick(), msi.getContactList(chuck, "norris"));
			printContactList(myself.getNick(), msi.getContactList(myself, "pallone"));
			
			
			try{
				//prova: invio messaggio a Chuck, utente amico sullo stesso server
				msi.forwardMessage(new Message("yabadabadooo!",chuck,myself));
				System.out.println("Forward ad utente amico consentito OK.");
			} catch (NotAllowedException e){
				System.out.println("ERRORE forward a utente amico non consentito: "+e);
			}
			
			System.out.println("Chuck scarica i messaggi");
			Vector<Message> mess1 = msi.downloadMsgs(chuck, "norris");
			printMessages(mess1,"Chuck");

			
			System.out.println("Chuck blocca l'utente marco");
			msi.blockContact(chuck, "norris", myself);
			printContactList(chuck.getNick(), msi.getContactList(chuck, "norris"));
			printContactList(myself.getNick(), msi.getContactList(myself, "pallone"));
			
			System.out.println("Chuck spedisce un messaggio a marco");
			msi.forwardMessage(new Message("ti ho bloccato!",myself, chuck));
			
			System.out.println("Marco spedisce un messaggio a chuck");
			try{
				msi.forwardMessage(new Message("che ti ho fatto?",chuck,myself));
				System.out.println("ERRORE: messaggio inviato da utente bloccato consentito");
			} catch (NotAllowedException e){
				System.out.println("OK: messaggio inviato da utente bloccato non consentito");
			}
			
			System.out.println("Chuck toglie il blocco all'utente marco rimettendolo come amico");
			msi.addToFriends(chuck, "norris", myself);
			printContactList(chuck.getNick(), msi.getContactList(chuck, "norris"));
			printContactList(myself.getNick(), msi.getContactList(myself, "pallone"));

			System.out.println("Marco invia un messaggio a Chuck");
			msi.forwardMessage(new Message("finalmente siamo tornati amici! non ti lascero' mai piu'.",chuck,myself));
			
			System.out.println("Chuck rimuove l'utente marco, che comunque puo' ancora scrivergli");
			msi.removeContact(chuck, "norris", myself);
			printContactList(chuck.getNick(), msi.getContactList(chuck, "norris"));
			printContactList(myself.getNick(), msi.getContactList(myself, "pallone"));

			System.out.println("Marco invia un messaggio a Chuck");
			msi.forwardMessage(new Message("Cosa credi, anche se mi rimuovi posso comunque scriverti.",chuck, myself));
			
			System.out.println("Chuck scarica i messaggi");
			Vector<Message> mess = msi.downloadMsgs(chuck, "norris");
			printMessages(mess,"Chuck");
			System.out.println("Chuck elimina i messaggi");
			msi.deleteDownloadedMsgs(chuck, "norris");
			
			
			System.out.println("Marco scarica i messaggi");
			Vector<Message> messM = msi.downloadMsgs(myself, "pallone");
			printMessages(messM,"Marco");
			System.out.println("Marco elimina i messaggi");
			msi.deleteDownloadedMsgs(myself, "pallone");
			System.out.println("Marco scarica i messaggi");
			messM = msi.downloadMsgs(myself, "pallone");
			printMessages(messM,"Marco");
			
			
			
			
			
			
			//prova invio a Mazinga, su un altro MS
			
			User mazinga = new User("mazinga",null,0,true,true);
			ms=bsInt.getMS(mazinga);
			MSForServletInterface msiMazinga = (MSForServletInterface) Naming.lookup("rmi://"+ms.getIP().getHostName()+":"+ms.getPort()+"/gossipMS");
			
			//prova: invio messaggio da Marco a Mazinga, utente non amico sconosciuto su un altro server
			msi.forwardMessage(new Message("raggi gamma!",mazinga,myself));
			System.out.println("OK forward ad utente sconosciuto su un altro server consentito.");

			try{
				printContactList(mazinga.getNick(), msiMazinga.getContactList(mazinga, ""));
				System.out.println("ERRORE: permessa getContactList di utente con password sbagliata");
			} catch (WrongPasswordException e){
				System.out.println("OK: non permessa getContactList di utente con password giusta");
			}

			System.out.println("Mazinga blocca Marco.");
			msiMazinga.blockContact(mazinga, "zeta", myself);
			printContactList(mazinga.getNick(), msiMazinga.getContactList(mazinga, "zeta"));

			try{
				//prova: invio messaggio a Mazinga, utente su un altro server che ha bloccato il mittente
				msi.forwardMessage(new Message("raggi beta!",mazinga,myself));
				System.out.println("ERRORE Forward ad utente bloccato consentito.");
			} catch (NotAllowedException e){
				System.out.println("OK forward a utente bloccato non consentito: "+e);
			}
			
			System.out.println("Mazinga aggiunge Marco agli amici.");
			msiMazinga.addToFriends(mazinga, "zeta", myself);
			printContactList(mazinga.getNick(), msiMazinga.getContactList(mazinga, "zeta"));
		
			
			try{
				//prova: invio messaggio a Mazinga, utente amico su un altro server
				msi.forwardMessage(new Message("raggi alfa!",mazinga,myself));
				System.out.println("Forward ad utente amico su altro server consentito OK.");
			} catch (NotAllowedException e){
				System.out.println("ERRORE forward a utente amico su altro server non consentito: "+e);
			}
			
						
			
		    
		} catch (Exception e){
			e.printStackTrace();
		}

		

	}
	
	private static void printMessages(Vector<Message> mess, String nick) {
		System.out.println("Messaggi di "+nick+"\nTotale messaggi: "+mess.size());
		for (int i=0; i<mess.size();i++){
			Message m = mess.get(i);
			System.out.println("Messaggio id "+i+"\nda: "+m.getFrom().getNick()+"\nper: "+m.getTo().getNick()+"\nSpedito: "+m.getDate()+"\nTesto: "+m.getText()+"\n");
		}
	}

	public static void printContactList(String nick, ContactList c){
		System.out.println("Lista contatti di "+nick.toUpperCase()+":");
		for (int i=0; i<c.getSize();i++){
			User u = c.getUser(i);
			System.out.println(u.getNick()+" "+u.isFriend());
		}
		
	}
}
